@@ -35,6 +35,8 @@ warnings.filterwarnings("ignore", "Accessed deprecated property",
                         DeprecationWarning)
 
 import distro_info
+import fnmatch
+import json
 import os
 import subprocess
 import sys
@@ -52,6 +54,7 @@ from .Dialogs import (DistUpgradeDialog,
                       HWEUpgradeDialog,
                       NeedRestartDialog,
                       NoUpdatesDialog,
+                      NoUpgradeForYouDialog,
                       PartialUpgradeDialog,
                       StoppedUpdatesDialog,
                       UnsupportedDialog,
@@ -62,7 +65,7 @@ from .Core.AlertWatcher import AlertWatcher
 from .Core.MyCache import MyCache
 from .Core.roam import NetworkManagerHelper
 from .Core.UpdateList import UpdateList
-from .Core.utils import get_dist
+from .Core.utils import get_arch, get_dist
 from .backend import (InstallBackend,
                       get_backend)
 
@@ -83,9 +86,13 @@ class UpdateManager(Gtk.Window):
         self.unity = UnitySupport()
         self.controller = None
         self.cache = None
+        self.ua_security_packages = []
         self.update_list = None
         self.meta_release = None
         self.hwe_replacement_packages = None
+        self.oem_metapackages = set()
+        self.duplicate_packages = []
+        self.arch = get_arch()
 
         # Basic GTK+ parameters
         self.set_title(_("Software Updater"))
@@ -219,7 +226,7 @@ class UpdateManager(Gtk.Window):
         self._start_pane(update_backend)
 
     def start_install(self, hwe_upgrade=False):
-        install_backend = get_backend(self, InstallBackend.ACTION_INSTALL)
+        install_backend = get_backend(self, InstallBackend.ACTION_PRE_INSTALL)
         if hwe_upgrade:
             for pkgname in self.hwe_replacement_packages:
                 try:
@@ -235,11 +242,54 @@ class UpdateManager(Gtk.Window):
         if self.cache is None:
             return
 
-        pane = self._make_available_pane(self.cache.install_count,
+        pane = self._make_available_pane(self.cache.install_count
+                                         + self.cache.del_count,
                                          os.path.exists(REBOOT_REQUIRED_FILE),
                                          cancelled_update, error_occurred)
         self._start_pane(pane)
 
+    def _check_oem_metapackages(self):
+        di = distro_info.UbuntuDistroInfo()
+        codename = get_dist()
+        lts = di.is_lts(codename)
+        if not lts:
+            return None
+        OEM_PATH = os.path.join(GLib.get_user_runtime_dir(),
+                                "ubuntu-drivers-oem.package-list")
+        if not os.path.exists(OEM_PATH):
+            return None
+
+        # Packages that aren't installed but apply to this system
+        with open(OEM_PATH, 'r') as f:
+            self.oem_metapackages |= set(f.read().splitlines())
+
+        # Packages that are already installed
+        for pkg in self.cache:
+            if fnmatch.fnmatch(pkg.name, 'oem-*-meta') \
+               and pkg.installed:
+                self.oem_metapackages.add(pkg)
+
+    def _get_ua_security_status(self):
+        self.ua_security_packages = []
+        try:
+            p = subprocess.Popen(['pro', 'security-status', '--format=json'],
+                                 stdout=subprocess.PIPE)
+        except OSError:
+            pass
+        else:
+            while p.poll() is None:
+                while Gtk.events_pending():
+                    Gtk.main_iteration()
+                time.sleep(0.05)
+            s = json.load(p.stdout)
+            for package in s.get('packages', []):
+                status = package.get('status', '')
+                if status == 'pending_attach':
+                    name = package.get('package', '')
+                    version = package.get('version', '')
+                    size = package.get('download_size', 0)
+                    self.ua_security_packages.append((name, version, size))
+
     def _make_available_pane(self, install_count, need_reboot=False,
                              cancelled_update=False, error_occurred=False):
         self._check_hwe_support_status()
@@ -310,6 +360,9 @@ class UpdateManager(Gtk.Window):
         if (self.meta_release.new_dist
                 and (self.options.check_dist_upgrades
                      or settings.get_boolean("check-dist-upgrades"))):
+            if self.arch == 'i386':
+                return NoUpgradeForYouDialog(self, self.meta_release,
+                                             self.arch)
             return DistUpgradeDialog(self, self.meta_release)
 
         return None
@@ -390,9 +443,23 @@ class UpdateManager(Gtk.Window):
                 Gtk.main_iteration()
         iterate()
 
+        self._check_oem_metapackages()
+
+        self._get_ua_security_status()
+
+        for pkgname in self.oem_metapackages:
+            try:
+                if not self.cache[pkgname].is_installed:
+                    self.cache[pkgname].mark_install()
+            except SystemError:
+                pass
+
         self.update_list = UpdateList(self)
         try:
-            self.update_list.update(self.cache, eventloop_callback=iterate)
+            self.update_list.update(self.cache, eventloop_callback=iterate,
+                                    duplicate_packages=self.duplicate_packages,
+                                    ua_security_packages=self.
+                                    ua_security_packages)
         except SystemError as e:
             header = _("Could not calculate the upgrade")
             desc = _("An unresolvable problem occurred while "
